import { Misc } from "./misc.js";

/* -------------------------------------------- */
/**
 * Mapping des types d'Item/Actor vers le nom d'affichage.
 * Par défaut, on prend le type avec la première lettre
 * majuscule, pas besoin d'ajouter tous les types.
 */
const typeDisplayName = {
  "competence": "Compétence",
  "herbe": "Plante",
  "ingredient": "Ingrédient",
  "queue": "Queue de Dragon",
  "ombre": "Ombre de Thanatos",
  "souffle": "Souffle de Dragon",
  "tete": "Tête de Dragon",
  "nourritureboisson": "Nourriture & boisson",
  "rencontresTMR": "Rencontre des TMR",
  "competencecreature": "Compétence de créature",
  "nombreastral": "Nombre astral",
  "casetmr": "Effet sur TMR",
  "recettealchimique": "Recette alchimique",
  "recettecuisine": "Recette de cuisine",
  "tarot": "Carte de tarot draconique",
  "tache": "Tâche",
  "meditation": "Méditation",
  "musique": "Mélodie",
  "chant": "Chanson",
  "creature": "Créature",
  "entite": "Entité",
  "vehicule": "Véhicule"
}

export class RddCompendiumOrganiser {
  static init() {
    Hooks.on('renderCompendium', async (pack, html, data) => RddCompendiumOrganiser.onRenderCompendium(pack, html, data))
  }

  static async onRenderCompendium(compendium, html, data) {
    console.log('onRenderCompendium', compendium, html, data);
    let pack = compendium.collection
    if (pack.metadata.system === 'foundryvtt-reve-de-dragon') {
      html.find('.directory-item').each((i, element) => {
        const entity = pack.get(element.dataset.documentId);
        const entityName = entity?.documentName;
        if (entityName === 'Actor' || entityName === 'Item') {
          const entityType = entity.data.type;
          const typeName = typeDisplayName[entityType] ?? Misc.upperFirst(entitytype);
          RddCompendiumOrganiser.insertEntityType(element, typeName);
        }
      });
    }
  }

  static insertEntityType(element, type) {
    element.children[1].insertAdjacentHTML('afterbegin', `<label>${type}: </label>`);
  }

}